/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2014  SEGGER Microcontroller GmbH & Co. KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.24 - Graphical user interface for embedded applications **
All  Intellectual Property rights  in the Software belongs to  SEGGER.
emWin is protected by  international copyright laws.  Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with the following terms:

The software has been licensed to  ARM LIMITED whose registered office
is situated at  110 Fulbourn Road,  Cambridge CB1 9NJ,  England solely
for  the  purposes  of  creating  libraries  for  ARM7, ARM9, Cortex-M
series,  and   Cortex-R4   processor-based  devices,  sublicensed  and
distributed as part of the  MDK-ARM  Professional  under the terms and
conditions  of  the   End  User  License  supplied  with  the  MDK-ARM
Professional. 
Full source code is available at: www.segger.com

We appreciate your understanding and fairness.
----------------------------------------------------------------------
File        : LCD_X.c
Purpose     : Port routines for LPC1800 SPI & RGB
----------------------------------------------------------------------
*/

#include "GUI.h"

/*********************************************************************
*
*       Hardware configuration
*
**********************************************************************
*/

#include "LPC18xx.h"
#include "SCU_LPC18xx.h"
#include "GPIO_LPC18xx.h"
#include "Driver_SPI.h"

/* GLCD pins:
   - LCD_VD19  : P4_1
   - LCD_VD3   : P4_2
   - LCD_VSYNC : P4_5
   - LCD_EN    : P4_6
   - LCD_DOTCLK: P4_7
   - LCD_VD11  : P4_9
   - LCD_VD10  : P4_10
   - LCD_BL_EN : P7_0  = GPIO3[8]
   - LCD_HSYNC : P7_6
   - LCD_VD12  : P8_3
   - LCD_VD7   : P8_4
   - LCD_VD6   : P8_5
   - LCD_VD5   : P8_6
   - LCD_VD4   : P8_7
   - LCD_VD23  : PB_0
   - LCD_VD22  : PB_1
   - LCD_VD21  : PB_2
   - LCD_VD20  : PB_3
   - LCD_VD15  : PB_4
   - LCD_VD14  : PB_5
   - LCD_VD13  : PB_6             */

#define     GLCD_PIN_NUMBER             21

/* GLCD GPIO definitions */
const GPIO_ID GLCD_GPIO[] = {
  { 3, 8 }          /* Only backlight control is used as GPIO pin */
};

/* GLCD pin definitions */
const PIN_ID GLCD_PIN[] = {
  {  4,  1, (SCU_CFG_MODE_FUNC5 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  4,  2, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  4,  5, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  4,  6, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  4,  7, (SCU_CFG_MODE_FUNC0 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  4,  9, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  4, 10, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  7,  0, (SCU_CFG_MODE_FUNC0 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  7,  6, (SCU_CFG_MODE_FUNC3 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  8,  3, (SCU_CFG_MODE_FUNC3 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  8,  4, (SCU_CFG_MODE_FUNC3 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  8,  5, (SCU_CFG_MODE_FUNC3 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  8,  6, (SCU_CFG_MODE_FUNC3 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  {  8,  7, (SCU_CFG_MODE_FUNC3 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  { 11,  0, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  { 11,  1, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  { 11,  2, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  { 11,  3, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  { 11,  4, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  { 11,  5, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)},
  { 11,  6, (SCU_CFG_MODE_FUNC2 | SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)}
};

/* SPI Interface: SSP0
 
   PINS: 
   - CS  = PF_1 (GPIO7[16])
   - SCK = PF_0 (SCK0)
   - SDO = PF_2 (MISO0)
   - SDI = PF_3 (MOSI0)
*/

/* SPI Driver */
extern ARM_DRIVER_SPI         Driver_SPI0;
#define ptrSPI              (&Driver_SPI0)

#define SDRAM_BASE_ADDR       0x28000000

/* SCU pin configuration definitions */
#define LCD_PIN_SET          (SCU_PIN_CFG_INPUT_FILTER_DIS | SCU_PIN_CFG_INPUT_BUFFER_EN | SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN)
#define LCD_NPR_SET          (SCU_PIN_CFG_HIGH_SPEED_SLEW_RATE_EN | SCU_PIN_CFG_PULLUP_DIS)

/* Clock Control Unit register bits */
#define CCU_CLK_CFG_RUN      (1 << 0)
#define CCU_CLK_CFG_AUTO     (1 << 1)
#define CCU_CLK_STAT_RUN     (1 << 0)



/*********************************************************************
*
*       Exported code
*
*********************************************************************
*/
/*********************************************************************
*
*       LCD_X_Init
*
* Purpose:
*   This routine should be called from your application program
*   to set port pins to their initial values
*/
void LCD_X_Init(void) {
  uint32_t i;

  /* Initialize and configure SPI */
  ptrSPI->Initialize  (NULL);
  ptrSPI->PowerControl(ARM_POWER_FULL);
  ptrSPI->Control     (ARM_SPI_MODE_MASTER | ARM_SPI_CPOL1_CPHA1 | ARM_SPI_MSB_LSB | ARM_SPI_SS_MASTER_SW | ARM_SPI_DATA_BITS(8), 10000000);

  LPC_CGU->BASE_LCD_CLK  = (1    << 11) |
                           (0x10 << 24) ; /* IDIVE is clock source            */

  /* Enable LCD clock */
  LPC_CCU1->CLK_M3_LCD_CFG   = CCU_CLK_CFG_AUTO | CCU_CLK_CFG_RUN;
  while (!(LPC_CCU1->CLK_M3_LCD_STAT & CCU_CLK_STAT_RUN));
 
  /* Enable GPIO Clock */
  GPIO_PortClock (1);

  /* Configure pins */
  for (i = 0; i < GLCD_PIN_NUMBER; i++)
    SCU_PinConfigure (GLCD_PIN[i].port,  GLCD_PIN[i].num,  GLCD_PIN[i].config_val);
  /* Configure backlight pin GPIO mode */
  GPIO_SetDir        (GLCD_GPIO[0].port, GLCD_GPIO[0].num, GPIO_DIR_OUTPUT);

  /* Configure LCD controller */
  LPC_RGU->RESET_CTRL0 = (1U << 16);
  while ((LPC_RGU->RESET_ACTIVE_STATUS0 & (1U << 16)) == 0);

  LPC_LCD->CTRL &= ~(1 << 0);           /* Disable LCD                        */
  LPC_LCD->INTMSK = 0;                  /* Disable all interrupts             */

  LPC_LCD->UPBASE = SDRAM_BASE_ADDR;

  LPC_LCD->TIMH  = (7    << 24) |       /* Horizontal back porch              */
                   (3    << 16) |       /* Horizontal front porch             */
                   (3    <<  8) |       /* Horizontal sync pulse width        */
                   (14   <<  2) ;       /* Pixels-per-line                    */
  LPC_LCD->TIMV  = (3    << 24) |       /* Vertical back porch                */
                   (2    << 16) |       /* Vertical front porch               */
                   (3    << 10) |       /* Vertical sync pulse width          */
                   (319  <<  0) ;       /* Lines per panel                    */
  LPC_LCD->POL   = (1    << 26) |       /* Bypass pixel clock divider         */
                   (239  << 16) |       /* Clocks per line: num of LCDCLKs    */
                   (1    << 13) |       /* Invert panel clock                 */
                   (1    << 12) |       /* Invert HSYNC                       */
                   (1    << 11) ;       /* Invert VSYNC                       */
  LPC_LCD->LE    = (1    << 16) |       /* LCDLE Enabled: 1, Disabled: 0      */
                   (9    <<  0) ;       /* Line-end delay: LCDCLK clocks - 1  */
  LPC_LCD->CTRL  = (1    << 11) |       /* LCD Power Enable                   */
                   (1    <<  5) |       /* 0 = STN display, 1: TFT display    */
                   (6    <<  1) ;       /* Bits per pixel: 16bpp (5:6:5)      */

  for (i = 0; i < 256; i++) {
    LPC_LCD->PAL[i] = 0;                /* Clear palette                      */
  }
  LPC_LCD->CTRL |= (1 <<  0);           /* LCD enable                         */

  /* Turn backlight on */
  GPIO_PinWrite (GLCD_GPIO[0].port, GLCD_GPIO[0].num, 1);
}

/*********************************************************************
*
*       LCD_X_ClrCS
*
* Purpose:
*   Sets the chip select pin to low
*/
void LCD_X_ClrCS(void) {
  ptrSPI->Control(ARM_SPI_CONTROL_SS, ARM_SPI_SS_ACTIVE);
}

/*********************************************************************
*
*       LCD_X_SetCS
*
* Purpose:
*   Sets the chip select pin to high
*/
void LCD_X_SetCS(void) {
  ptrSPI->Control(ARM_SPI_CONTROL_SS, ARM_SPI_SS_INACTIVE);
}

/*********************************************************************
*
*       LCD_X_WriteM
*
* Purpose:
*   Writes multiple bytes to controller
*/
void LCD_X_WriteM(U8 * pData, int NumBytes) {
  ptrSPI->Send(pData, NumBytes);
  while (ptrSPI->GetStatus().busy);
}

/*********************************************************************
*
*       LCD_X_ReadM
*
* Purpose:
*   Reads multiple bytes from the controller
*/
void LCD_X_ReadM(U8 * pData, int NumBytes) {
  ptrSPI->Receive(pData, NumBytes);
  while (ptrSPI->GetStatus().busy);
}

/*************************** End of file ****************************/
